drop table depts
/


CREATE TABLE DEPTs
       (DEPTNO NUMBER(2),
        DNAME VARCHAR2(14),
        LOC VARCHAR2(13) );


INSERT INTO depts VALUES (10, 'ACCOUNTING', 'NEW YORK');
INSERT INTO depts VALUES (20, 'RESEARCH',   'DALLAS');
INSERT INTO depts VALUES (30, 'SALES',      'CHICAGO');
INSERT INTO depts VALUES (40, 'OPERATIONS', 'BOSTON');
commit;


create or replace package rss_dml
is

procedure feed(p_table_name in varchar2);

end;
/

grant execute on rss_dml to public
/


create or replace package body rss_dml
is

procedure channel
( p_title in varchar2
, p_description in varchar2
) is
begin
  htp.p(' 
	<channel>
<title>'||p_title||'</title>
<description>'||p_description||'</description>
<pubDate>'||to_char(systimestamp)||'</pubDate>
<language>en</language>');

end channel;

procedure item
( p_title in varchar2
, p_pubDate in varchar2
, p_guid in varchar2
, p_description in varchar2 
) is
begin
    htp.p('<item>
    	  <title>'||p_title||'</title>
          <pubDate>'||p_pubDate||'</pubDate>
          <creator>John Doe</creator>
          <category>DML Modifications</category>
          <guid>'||p_guid||'</guid>
	  <description>'||p_description||'
           </description>
       </item>');
end item;

procedure feed(p_table_name in varchar2)
is
begin
  owa_util.mime_header('text/xml', FALSE); 
  owa_util.http_header_close; 
  htp.p('<?xml version=''1.0'' ?>');
  htp.p('<rss version="2.0">');
  channel
  ( p_title => p_table_name||' DML Events'
  , p_description => 'Overview of the most recent DML operations in our database on table '||p_table_name
  );
  for o in (select versions_starttime
            ,      versions_endtime
            ,      versions_xid
            ,      nvl(versions_operation,'I') versions_operation
            ,      dname 
            ,      lead(dname) over (partition by deptno order by versions_starttime desc nulls last) previous_dname
            ,      deptno
            ,      loc
            ,      lead(loc) over (partition by deptno order by versions_starttime desc nulls last) previous_loc
            from   depts versions between timestamp minvalue and maxvalue
            order 
            by     versions_starttime desc nulls last
           ) loop
      if nvl(o.previous_loc,'X') <> nvl(o.loc,'X')
         or
         nvl(o.previous_dname,'X') <> nvl(o.dname,'X')
         or
         o.versions_operation in ('I','D')         
      then
        item
        ( p_title => 'Change History for Department '||o.deptno
        , p_pubDate => to_char(nvl(o.versions_starttime,o.versions_endtime),'DAY, MON, DD, YYYY hh24:mi' )
        , p_guid => o.deptno||'-'||o.versions_xid
        , p_description => 'Department '||o.deptno||' was '||
	  case o.versions_operation
	  when 'I' then 'inserted'
	  when 'U' then 'updated'
	  when 'D' then 'deleted'
	  end
	  ||':	'||
	case 
	when nvl(o.previous_loc,'X') <> nvl(o.loc,'X')
	then '* The location was changed from '||o.previous_loc||' to '||o.loc
	end
	||
	case 
	when nvl(o.previous_dname,'X') <> nvl(o.dname,'X')
	then '* The dname was changed from '||o.previous_dname||' to '||o.dname	
	end
        );
     end if;
   end loop;
   htp.p('</channel>');
   htp.p('</rss>');

end;

end;
/

cl scr
set feedback on
set echo on




update depts
set    loc = 'WASH DC'
where  deptno = 30
/
commit;

prompt Check the RSS Feed

pause 

update depts
set    dname = 'QUIZ'
where  deptno = 20
/
commit;

pause 

update depts
set    dname = 'CONSULTING'
where  deptno = 10
/
commit;

pause 

update depts
set    dname = 'CATERING'
,      loc = 'VENICE'
where  deptno = 30
/
commit;

pause 

update depts
set    dname = 'QUALITY'
where  deptno = 40
/
commit;

pause 

insert into depts
( deptno, dname, loc)
values
( 50 ,'EDUCATION', 'AUSTIN')
/
commit;

pause 



delete 
from depts 
where deptno = 50
/
commit;

pause 


prompt URL = http://localhost:7777/rob/scott.rss_dml.feed?p_table_name=DEPTS